#include "largefilereadertask.h"

LargeFileReaderTask::LargeFileReaderTask( const QString& fileName, quint64 chunkSizeKb ) :
    QObject( NULL ), m_file( fileName ), m_chunkSizeKb( chunkSizeKb ), m_cancelledMarker( false ) {

    if( m_chunkSizeKb < MIN_CHUNK_SIZE_KB ) {
        m_chunkSizeKb = MIN_CHUNK_SIZE_KB;
    }
}

LargeFileReaderTask::~LargeFileReaderTask() {
}

void LargeFileReaderTask::run() {
    if( !m_file.open( QIODevice::ReadOnly ) ) {
        emit readingFinished( RESULT_FAILED );
        return;
    }

    QByteArray data;
    QByteArray chunk;
    do {
        if( m_cancelledMarker.testAndSetAcquire( true, true ) ) {
            emit readingFinished( RESULT_CANCELLED );
            return;
        }

        try {
            chunk = m_file.read( m_chunkSizeKb * 1024 );
            data.append( chunk );
        } catch( ... ) {
            emit readingFinished( RESULT_FAILED );
            return;
        }

        emit progressChanged( data.size(), m_file.size() );
    } while( !chunk.isEmpty() );

    if( m_file.error() != QFile::NoError ) {
        emit readingFinished( RESULT_FAILED );
        return;
    }

    emit readingFinished( RESULT_OK, data );
}

void LargeFileReaderTask::cancel() {
    m_cancelledMarker.fetchAndStoreAcquire( true );
}
